@@ -15,22 +15,24 @@
  * ------------------------------------------------------------------------------
  */
 
+use sha2::{Digest, Sha256, Sha512};
+
 pub fn sha256_digest_str(item: &str) -> String {
-    hex::encode(openssl::sha::sha256(item.as_bytes()))
+    hex::encode(sha256_digest_strs(&[item.to_string()]))
 }
 
 pub fn sha256_digest_strs(strs: &[String]) -> Vec<u8> {
-    let mut hasher = openssl::sha::Sha256::new();
+    let mut hasher = Sha256::new();
     for item in strs {
         hasher.update(item.as_bytes());
     }
     let mut bytes = Vec::new();
-    bytes.extend(hasher.finish().iter());
+    bytes.extend(hasher.finalize().into_iter());
     bytes
 }
 
 pub fn sha512_digest_bytes(item: &[u8]) -> Vec<u8> {
-    let mut bytes: Vec<u8> = Vec::new();
-    bytes.extend(openssl::sha::sha512(item).iter());
-    bytes
+    let mut hasher = Sha512::new();
+    hasher.update(item);
+    hasher.finalize().to_vec()
 }
