@@ -21,6 +21,8 @@ use std::collections::{HashMap, HashSet};
 use std::iter::Peekable;
 use std::sync::{Arc, RwLock};
 
+use log::{debug, error, warn};
+
 use crate::journal::block_store::{
     BatchIndex, BlockStoreError, IndexedBlockStore, TransactionIndex,
 };
@@ -312,7 +314,7 @@ impl BlockManagerState {
 
         gauge!(
             "block_manager.BlockManager.pool_size",
-            block_by_block_id.len() as i64
+            block_by_block_id.len() as f64
         );
 
         Ok(())
